﻿using System.Threading.Tasks;
using Databricks.Cli;
using Spectre.Console.Cli;

namespace dbc
{
   class Program
   {
      static async Task<int> Main(string[] args)
      {
         // required for emoji to render
         System.Console.OutputEncoding = System.Text.Encoding.UTF8;

         var app = new CommandApp();
         app.Configure(config =>
         {
#if DEBUG
            config.PropagateExceptions();
#endif

            config.AddBranch<BaseSettings>("query", query =>
            {
               query
                  .AddCommand<ListQueriesCommand>("list")
                  .WithDescription("lists all queries");

               query
                  .AddCommand<GetQueryCommand>("get")
                  .WithDescription("get query details");

               query
                  .AddCommand<DeleteQueryCommand>("delete")
                  .WithDescription("delete query or queries");

               query
                  .AddCommand<BackupQueriesCommand>("backup")
                  .WithDescription("backup queries to local files");
            });

            config.AddBranch<BaseSettings>("dash", query =>
            {
               query
                  .AddCommand<ListDashboardsCommand>("list")
                  .WithDescription("list all dashboards");

               query
                  .AddCommand<BackupDashboardCommand>("backup")
                  .WithDescription("backup dashboards to local files");
            });

            config.AddBranch<BaseSettings>("cluster", query =>
            {
               query
                  .AddCommand<ListClustersCommand>("list")
                  .WithDescription("list all clusters");

               query
                  .AddCommand<StartClusterCommand>("start")
                  .WithDescription("start cluster by id or name");

               query
                  .AddCommand<StopClusterCommand>("stop")
                  .WithDescription("stop cluster by id or name");

            });

            config.AddBranch<BaseSettings>("job", query =>
            {
               query
                  .AddCommand<ListJobsCommand>("list")
                  .WithDescription("list all jobs");

               query
                  .AddCommand<StartJobCommand>("start")
                  .WithDescription("start a job by id or name");

               query
                  .AddCommand<StopJobCommand>("stop")
                  .WithDescription("stop a job by id or name");

               query
                  .AddCommand<UpsertJobCommand>("upsert")
                  .WithDescription("create or update job given it's JSON definition");
            });

            config.AddBranch<BaseSettings>("exec", query =>
            {
               query
                  .AddCommand<ExecPythonCommand>("python")
                  .WithDescription("execute Python code");

               query
                  .AddCommand<ExecScalaCommand>("scala")
                  .WithDescription("execute Scala code");

               query
                  .AddCommand<ExecSqlCommand>("sql")
                  .WithDescription("execute SQL code");
            });

            config.AddBranch<BaseSettings>("scim", query =>
            {
               query
                  .AddCommand<ScimListUsersCommand>("list-users")
                  .WithDescription("List users. When exporting to CSV, the group column is exploded.");
            });
         });

         return await app.RunAsync(args);
      }
   }
}